from __future__ import annotations

from collections.abc import Hashable
from pathlib import Path
from typing import Any, Protocol


class AbstractConnection(Protocol):
    def get(self, key: Hashable, default: Any, /) -> Any:
        raise NotImplementedError

    async def set(self, key: Hashable, value: Any, /) -> None:
        raise NotImplementedError


class AbstractDbManager(Protocol):
    async def __aenter__(self) -> AbstractConnection:
        raise NotImplementedError

    async def __aexit__(self, et, ev, tb, /):
        raise NotImplementedError


class AbstractDbFactory(Protocol):
    def from_path(self, path: Path) -> AbstractDbManager:
        raise NotImplementedError
